package be.iminds.ilabt.jfed.ui.cli2.instruction;

import be.iminds.ilabt.jfed.experiment.setup.config.ESpec;
import be.iminds.ilabt.jfed.experiment.setup.config.Slice;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class SliceActionInstruction extends InstructionWithSlice {
    public enum SliceAction {
        LinkTest(), ConnectivityTest("runConnectivityTest"), RerunESpec("espec", "runEspec");

        private final Set<String> names;

        SliceAction(@Nullable String... names) {
            this.names = names == null ?
                    Collections.singleton(canonized(name())) :
                    Stream.concat(Arrays.stream(names), Stream.of(name()))
                            .map(SliceAction::canonized)
                            .collect(Collectors.toSet());
        }

        public static String canonized(String value) {
            return value.trim().toLowerCase().replaceAll(" _-", "");
        }

        public boolean matches(@Nonnull String value) {
            return names.contains(canonized(value));
        }

        @JsonCreator
        public static SliceAction parse(@Nonnull String value) {
            return Arrays.stream(SliceAction.values())
                    .filter(m -> m.matches(value))
                    .findAny()
                    .orElseThrow(() -> new IllegalArgumentException("SliceAction not known: "+value));
        }
    }

    @Nonnull private final SliceAction sliceAction;
    @Nullable private final ESpec espec;

    @JsonCreator
    public SliceActionInstruction(@JsonProperty("action") @Nonnull Action action,
                                  @JsonProperty("user") @Nullable User user,
                                  @JsonProperty("actionOutputs") @Nullable List<ActionOutput> actionOutputs,
                                  @JsonProperty("debugOutputs") @Nullable List<DebugOutput> debugOutputs,
                                  @JsonProperty("callOutputs") @Nullable List<CallOutput> callOutputs,
                                  @JsonProperty("proxy") @Nullable ProxySettings proxy,
                                  @JsonProperty("slice") @Nullable Slice slice,
                                  @JsonProperty("sliceAction") @Nonnull SliceAction sliceAction,
                                  @JsonProperty("espec") @Nullable ESpec espec) {
        super(action, user, actionOutputs, debugOutputs, callOutputs, proxy, slice);
        if (sliceAction == null)
            throw new IllegalArgumentException("sliceAction is required");
        this.sliceAction = sliceAction;
        this.espec = espec;
    }

    private SliceActionInstruction(boolean demo) {
        super(Action.SLICEACTION, demo, true);
        this.sliceAction = demo ? SliceAction.RerunESpec : SliceAction.RerunESpec;
        this.espec = demo ? new ESpec(ESpec.ESpecSource.PROVIDE_DIR, "/tmp/my-espec/", null) : null;
    }

    @JsonProperty
    @Nonnull
    public SliceAction getSliceAction() {
        return sliceAction;
    }

    @JsonProperty("espec")
    @Nullable
    public ESpec getEspec() {
        return espec;
    }

    @JsonIgnore
    @Nonnull
    @Override
    public Action getActionOfClass() {
        return Action.SLICEACTION;
    }
}
